from django.db import models
from django.http import JsonResponse

from wagtail.models import Page, Orderable
from wagtail.search import index
from wagtail.admin.panels import FieldPanel, InlinePanel, MultiFieldPanel, FieldRowPanel
from wagtail.fields import StreamField
from wagtail.snippets.models import register_snippet
from wagtail.fields import RichTextField

from django_extensions.db.fields import AutoSlugField

from modelcluster.fields import ParentalKey
from modelcluster.contrib.taggit import ClusterTaggableManager
from modelcluster.models import ClusterableModel

from taggit.models import Tag as TaggitTag
from taggit.models import TaggedItemBase
from taggit.models import TagBase, ItemBase

# from core.blocks import CarouselBlock, WebStoryBlock
from wagtailmodelchooser import register_model_chooser
from django.contrib.auth.models import User
from wagtail.contrib.routable_page.models import RoutablePageMixin, path, re_path
import datetime
# Create your models here.
from wagtail_color_panel.fields import ColorField
from wagtail_color_panel.edit_handlers import NativeColorPanel
from wagtail.admin.panels import TabbedInterface, TitleFieldPanel, ObjectList
from hitcount.models import HitCountMixin
from django.db.models import Q
import time
import calendar
from django_countries.fields import CountryField
from .blocks import *
from Service.models import SubCategory, Category
from wagtailmetadata.models import MetadataPageMixin


class TagColor(models.Model):
    bgColor = ColorField(
        default="#FFFFFF", verbose_name="BG Color", blank=True, null=True)
    fgColor = ColorField(
        default="#000000", verbose_name="FG Color", blank=True, null=True)

    class Meta:
        abstract = True


CO_TYPE_LIST = [
    ("Self", "Self"),
    ("Vendor", "Vendor"),
    ("Brand", "Brand"),
]
CO_ADDR_TYPE_LIST = [
    ("Office", "Office"),
    ("Branch", "Branch"),
    ("Warehouse", "Warehouse"),
]


@register_snippet
class Company(ClusterableModel):
    coType = models.CharField(max_length=250, blank=True, null=True,
                              verbose_name="Company Type", choices=CO_TYPE_LIST)
    coName = models.CharField(
        max_length=250, blank=True, null=True, verbose_name="Name")
    coWeb = models.CharField(max_length=250, blank=True,
                             null=True, verbose_name="Website")
    coContact = models.CharField(
        max_length=250, blank=True, null=True, verbose_name="Contact")
    coEmail = models.CharField(
        max_length=250, blank=True, null=True, verbose_name="Email")
    createdBy = models.ForeignKey(
        User, on_delete=models.SET_NULL, null=True, blank=True, verbose_name="Created By")
    panels = [
        MultiFieldPanel([
            FieldRowPanel([
                FieldPanel('coType'),
                FieldPanel('coName'),

            ]),
            FieldRowPanel([
                FieldPanel('coWeb'),
                FieldPanel('coContact'),

            ]),
            FieldRowPanel([
                FieldPanel('coEmail'),
                FieldPanel('createdBy'),

            ]),
            InlinePanel('company_address', label="Address"),
            InlinePanel('company_brands', label="Brand")

        ]),
    ]
    address = [
        InlinePanel('company_address', label="Address"),
    ]
    brand = [
        InlinePanel('company_brands', label="Brand")
    ]
    # edit_handler = TabbedInterface([
    #     ObjectList(panels, heading='Content'),
    #     # ObjectList(address, heading='Address'),
    #     # ObjectList(brand, heading='Brand'),
    #     # ObjectList(Page.settings_panels, heading='Settings'), # The default settings are now displayed in the sidebar but need to be in the `TabbedInterface`.
    # ])

    def __str__(self):
        return self.coName

    class Meta:
        verbose_name_plural = "Company"


class CompanyAddr(Orderable):
    com = ParentalKey(Company, related_name="company_address")
    coAddrTypeIcon = models.ForeignKey(
        'wagtailimages.Image', on_delete=models.CASCADE, related_name='+', verbose_name="Icon", blank=True, null=True
    )
    coAddrTypeIconText = models.CharField(
        max_length=250, blank=True, null=True, verbose_name="Text Icon")
    coAddrType = models.CharField(
        max_length=250, blank=True, null=True, verbose_name="Address Type", choices=CO_ADDR_TYPE_LIST)
    coAddrFull = models.TextField(
        blank=True, null=True, verbose_name="Address")
    coAddrCountry = CountryField(blank=True, null=True, verbose_name="Country")
    coAddrPin = models.CharField(
        max_length=250, blank=True, null=True, verbose_name="Pin")
    coAddrLoc = models.URLField(
        blank=True, null=True, verbose_name="Location Map URL")
    coAddrLat_map = models.FloatField(
        blank=True, null=True, verbose_name="Map Lat")
    coAddrLog_map = models.FloatField(
        blank=True, null=True, verbose_name="Map Long")
    createdBy = models.ForeignKey(
        User, on_delete=models.SET_NULL, null=True, blank=True, verbose_name="Created By")

    panels = [
        MultiFieldPanel([
            FieldRowPanel([
                FieldPanel('coAddrTypeIcon'),
                FieldPanel('coAddrTypeIconText'),
            ]),
            FieldPanel('coAddrType'),
            FieldPanel('coAddrFull'),
            FieldRowPanel([
                FieldPanel('coAddrCountry'),

                FieldPanel('coAddrPin'),
            ]),
            FieldPanel('coAddrLoc'),
            FieldRowPanel([
                FieldPanel('coAddrLat_map'),
                FieldPanel('coAddrLog_map'),
                FieldPanel('createdBy'),
            ])

        ])
    ]

    class Meta:
        verbose_name_plural = "Company Address"


@register_snippet
@register_model_chooser
class Brand(Orderable, TagColor):
    coName = ParentalKey(Company, related_name="company_brands")
    coBrandName = models.CharField(
        max_length=100, null=True, verbose_name="Name")
    coBrandImg = models.ForeignKey(
        "wagtailimages.Image",
        on_delete=models.SET_NULL,
        null=True,
        blank=True,
        related_name="+",
        verbose_name="Image"
    )
    coBrandImgText = models.CharField(
        max_length=150, null=True, verbose_name="Image Text")
    url = models.URLField(blank=True, null=True)
    aliasName = models.CharField(max_length=150, blank=True, null=True)
    panels = [
        MultiFieldPanel([
            FieldRowPanel([
                FieldPanel('coName'),
                FieldPanel('coBrandName'),
            ]),
            FieldRowPanel([
                FieldPanel('coBrandImg'),
                FieldPanel('coBrandImgText'),
            ]),
            FieldRowPanel([
                FieldPanel('url'),
                FieldPanel('aliasName'),
            ]),
            FieldRowPanel([
                NativeColorPanel('bgColor'),
                NativeColorPanel('fgColor'),

            ]),
        ])
    ]

    def __str__(self):
        return f'{self.coBrandName}'

    class Meta:
        verbose_name_plural = "Brand"
        ordering = ["-id"]


@register_snippet
@register_model_chooser
class Product(TagColor):
    coProdName = models.CharField(
        max_length=100, null=True, verbose_name="Product Name")
    coProdBrand = models.ForeignKey(
        Brand, on_delete=models.SET_NULL, null=True, blank=True, verbose_name="Brand")
    coProdCat = models.ForeignKey(
        SubCategory, on_delete=models.SET_NULL, null=True, blank=True, verbose_name="Category")
    coProdTechStack = RichTextField(
        blank=True, null=True, verbose_name="Technology Stack")
    coProdReq = RichTextField(blank=True, null=True,
                              verbose_name="Product Requirement")
    coProdDescription = RichTextField(
        blank=True, null=True, verbose_name="Description")
    coSupport = models.CharField(
        max_length=250, blank=True, null=True, verbose_name="Support")
    coUsers = models.PositiveIntegerField(
        blank=True, null=True, verbose_name="No of users")
    # coName = models.ForeignKey(
    #     Company, on_delete=models.SET_NULL, null=True, blank=True)
    parent = models.ForeignKey(
        'self', on_delete=models.PROTECT, null=True, blank=True, related_name='sub_product')

    def __str__(self):
        return f'{self.coProdName}'

    panels = [
        FieldPanel('coProdName'),
        MultiFieldPanel([
            FieldRowPanel([
                FieldPanel('coProdBrand'),
                FieldPanel('coProdCat'),
            ])
        ]),
        FieldPanel('coProdTechStack'),
        FieldPanel('coProdReq'),
        FieldPanel('coProdDescription'),
        FieldPanel('coSupport'),
        MultiFieldPanel([
            FieldRowPanel([
                FieldPanel('coUsers'),
                # FieldPanel('coName'),
                FieldPanel('parent'),
            ]),
            FieldRowPanel([
                NativeColorPanel('bgColor'),
                NativeColorPanel('fgColor'),

            ]),
        ]),
    ]

    class Meta:
        verbose_name_plural = 'Product'
        ordering = ["-id"]


@register_snippet
class BlogTag(TagBase):
    bgColor = ColorField(
        default="#FFFFFF", verbose_name="BG Color", blank=True, null=True)
    fgColor = ColorField(
        default="#000000", verbose_name="FG Color", blank=True, null=True)
    panels = [
        MultiFieldPanel([
            FieldRowPanel([

                FieldPanel('name'),
                FieldPanel('slug'),
            ]),
            FieldRowPanel([

                NativeColorPanel('bgColor'),
                NativeColorPanel('fgColor'),
            ])
        ])
    ]

    class Meta:
        verbose_name = "blog tag"
        verbose_name_plural = "blog tags"


class BlogPageTag(ItemBase):
    tag = models.ForeignKey(
        BlogTag, related_name="tagged_blogs", on_delete=models.CASCADE
    )
    content_object = ParentalKey(
        'BlogPage',
        related_name='tagged_items',
        on_delete=models.CASCADE
    )


# @register_snippet
class Tag(TaggitTag):
    class Meta:
        proxy = True


class BlogIndexPage(RoutablePageMixin,MetadataPageMixin, Page):
    intro = RichTextField(blank=True)
    banner = StreamField([
        ('banner', CarouselBlock()),
    ], use_json_field=True, blank=True, null=True, max_num=1)
    # template = "home/home_page.html"

    parent_page_type = [
        'Core.WebPages',
    ]
    subpage_types = [
        'Blog.BlogPage', 
        'Blog.TutorialPage', 
    ]

    content_panels = Page.content_panels + [
        FieldPanel('intro'),
        FieldPanel('banner')
    ]

    # def get_children(self):
    #     qs = super().get_children()
    #     # qs = BlogPage.objects.all()
    #     qs = qs.order_by('-last_published_at')
    #     return qs

    def get_child_blogs(self):
        qs = BlogPage.objects.all()
        qs = qs.order_by('-last_published_at')
        return qs

    def get_recent_blogs(self):
        max_count = 5  # max count for displaying post
        return BlogPage.objects.all().order_by('-first_published_at')[:max_count]

    def get_trending_blogs(self):
        max_count = 5  # max count for displaying post
        return BlogPage.objects.all().order_by('-blPoViewCount')[:max_count]

    def get_tags(self):
        return BlogTag.objects.all().order_by('-id')

   
    # def get_cat_archives(self):
    #     catObj = CategoryPage.objects.filter(type="Category").order_by('-id')
    #     catArchives = []
    #     for cat in catObj:
    #         blogs=BlogPage.objects.all()
    #         catblogs=[]
    #         for blog in blogs:
    #             for blcat in blog.blCategory:
    #                 if blcat.value.id==cat.id:
    #                     catblogs.append((blog))
    #         catArchives.append((cat,len(catblogs)))
    #                     # catArchives.append((cat, BlogPage.objects.filter( product__coProdCat=sc).count()))

    #     #     subCatObj = SubCategory.objects.filter(catName=cat).order_by("-id")
    #     #     print("cat=", cat)
    #     #     scArch = []
    #     #     for sc in subCatObj:
    #     #         print("sc=", sc)
    #     #         scArch.append((sc.scName, BlogPage.objects.filter(
    #     #             product__coProdCat=sc).count()))
    #     #     catArchives.append((cat.catName, scArch))
    #     return catArchives

    # def get_cat_archives(self):
    #     catObj = Category.objects.order_by('-id')
    #     catArchives = []
    #     for cat in catObj:
    #         subCatObj = SubCategory.objects.filter(catName=cat).order_by("-id")
    #         print("cat=", cat)
    #         scArch = []
    #         for sc in subCatObj:
    #             print("sc=", sc)
    #             scArch.append((sc.scName, BlogPage.objects.filter(
    #                 product__coProdCat=sc).count()))
    #         # catArchives.append((cat.catName))
    #         catArchives.append((cat.catName, scArch))
    #     return catArchives

    @path('tag/<tag>/')
    def post_by_tag(self, request, tag, *args, **kwargs):
        self.posts = BlogPage.objects.live().filter(tags__slug=tag)
        return self.render(request, context_overrides={
            'title': "Blogs for <b>"+tag+"</b>",
            'posts': self.posts,
        })

    @path('category/<str:slug>/')
    def post_by_category(self, request, slug, *args, **kwargs):
        # events = BlogPage.objects.live().filter(Q(product__coProdCat__scSlug=slug) |
        #                                         Q(product__coProdCat__catName__catSlug=slug))
        blogsList = []
        for blog in BlogPage.objects.all():
            for blCat in blog.blCategory:
                if blCat.value.slug == slug:
                    blogsList.append(blog)
        return self.render(request, context_overrides={
            'title': "Blogs for <b>" + slug+"</b>",
            'posts': blogsList,
        })

    @path('<int:year>/<int:month>/')
    def post_by_year_month(self, request, year=None, month=None, *args, **kwargs):
        events = BlogPage.objects.live().filter(
            first_published_at__year=year, first_published_at__month=month)
        # return self.render(request)
        print("Blogs=", events)
        return self.render(request, context_overrides={
            'title': "Blogs for <b>"+calendar.month_name[month]+", "+str(year)+"</b>",
            'posts': events,
        })


# Article
class BlogPage(MetadataPageMixin,Page, HitCountMixin):
    blPoTitleTemplate = models.CharField(max_length=200, blank=True, null=True, choices=[(
        'Single Row', 'Single Row'), ('Multi Row', 'Multi Row')], verbose_name="Title Template", default="Multi Row")
    customTitle=RichTextField(blank=True, null=True,verbose_name="Custom Title")
    titleImg = models.ForeignKey(
        'wagtailimages.Image', on_delete=models.PROTECT, related_name='+', blank=True, null=True, verbose_name="Title Image"
    )
    shortDesc = RichTextField(blank=True, null=True,
                              verbose_name="Short Description")

    blCategory = StreamField([
        ('category', ModelChooserBlock(Category)),
    ], use_json_field=True, blank=True, null=True)

    blbrands = StreamField([
        ('brand', ModelChooserBlock(Brand)),
    ], use_json_field=True, blank=True, null=True)

    # blSubCategory = StreamField([
    #     ('category', ModelChooserBlock(CategoryPage)),
    # ], use_json_field=True, blank=True, null=True)

    product = models.ForeignKey(
        Product, on_delete=models.SET_NULL, null=True, blank=True)

    tags = ClusterTaggableManager(through=BlogPageTag, blank=True)
    products = StreamField([
        ('blog', PageChooserBlock('Blog.BlogPage')),
        ('product', ModelChooserBlock(Product)),
    ], use_json_field=True, blank=True, null=True)

    body = StreamField([
        ('carousel', CarouselBlock()),
        ('section', SectionBlock()),
        ('row', RowBlock()),

    ], use_json_field=True, blank=True, null=True)
    blPoBanner = StreamField([
        ('banner', CarouselBlock()),
    ], use_json_field=True, blank=True, null=True, max_num=1)

    blogStory = StreamField([
        ('web_story', WebStoryBlock()),
    ], use_json_field=True, blank=True, null=True, max_num=1)

    blogSocial = StreamField([
        ('social_links', BlogSocialBlock()),
    ], use_json_field=True, blank=True, null=True)
    blogGallery = StreamField([
        ('gallery', BlogGalleryBlock()),
    ], use_json_field=True, blank=True, null=True)
    blPoViewCount = models.PositiveIntegerField(
        blank=True, null=True, default=0)
    blPoLikeCount = models.PositiveIntegerField(
        blank=True, default=0)
    blPoReadingTime = models.PositiveIntegerField(
        blank=True, null=True, default=2)
    showFooter = models.BooleanField(
        blank=True, null=True, default=False, verbose_name="Show Footer")
    blPoCommentLock = models.BooleanField(blank=True, null=True, default=False)
    blPoCommentLockMsg = models.CharField(
        max_length=200, blank=True, null=True)
    blPoRelatedTemplate = models.CharField(max_length=200, blank=True, null=True, choices=[(
        'Links', 'Links'), ('Img One Column', 'Img One Column'), ('Img Two Column', 'Img Two Column')], default="Img Two Column")
    content_panels = Page.content_panels + [
        FieldPanel('customTitle'),

        FieldPanel('blCategory', heading="Category"),
        FieldPanel('blbrands', heading="Brands"),
        FieldPanel('products'),
        MultiFieldPanel([
            FieldRowPanel([
                FieldPanel('blPoTitleTemplate'),
                # FieldPanel('blProduct'),
                # FieldPanel('product'),
                FieldPanel('titleImg'),
            ])
        ]),
        # FieldPanel('catVersion'),
        FieldPanel('shortDesc'),
        FieldPanel('tags'),
        FieldPanel('body'),
    ]
    promote_panels = Page.promote_panels + [
        FieldPanel('blPoCommentLock', heading="Lock Comment"),
        FieldPanel('blPoCommentLockMsg', heading="Message on Lock Comment"),
        FieldPanel('blPoReadingTime', heading="Reading Time"),
        FieldPanel('showFooter'),
        FieldPanel('blPoViewCount', heading="Views"),
        FieldPanel('blPoLikeCount', heading="Likes"),
    ]
    banner_panels = [
        FieldPanel('blPoBanner', heading="Banner")
    ]
    story_panels = [
        FieldPanel('blogStory', heading="Web Story")
    ]
    social_panels = [
        FieldPanel('blogSocial', heading="Social")
    ]
    gallery_panels = [
        FieldPanel('blogGallery', heading="Gallery")
    ]
    comments_panels = [
        InlinePanel('blog_comments', label="Comments")
    ]
    re_panels = [
        FieldPanel('blPoRelatedTemplate', heading="Template"),

        InlinePanel('blog_reLinks', label="Related Link")
    ]
    edit_handler = TabbedInterface([
        ObjectList(content_panels, heading='Content'),
        ObjectList(promote_panels, heading='Promote'),
        ObjectList(banner_panels, heading='Banner'),
        ObjectList(story_panels, heading='Story'),
        ObjectList(social_panels, heading='Social'),
        ObjectList(gallery_panels, heading='Gallery'),
        ObjectList(re_panels, heading='Related'),
        ObjectList(comments_panels, heading='Comments'),
        # ObjectList(Page.settings_panels, heading='Settings'), # The default settings are now displayed in the sidebar but need to be in the `TabbedInterface`.
    ])
    search_fields = Page.search_fields + [  # Inherit search_fields from Page
        index.SearchField('title'),
        # index.SearchField('category'),
        index.SearchField('tags'),
        index.SearchField('body'),
        index.FilterField('title'),
        # index.FilterField('category'),
        index.FilterField('tags'),
        index.FilterField('body'),
    ]

    def __str__(self):
        return str(self.title)

    def get_cat(self):
        print("Category=", self.blCategory)
        # self.blPoLikeCount = self.blPoLikeCount+int(1)
        # self.save()  # note: you may need to ensure you call save to update data

    def archives(self):
        """Make a list of months to show archive links."""
        if not BlogPage.objects.count():
            return []

        # set up vars
        year, month = time.localtime()[:2]
        first = BlogPage.objects.order_by("first_published_at")[0]
        fyear = first.first_published_at.year
        fmonth = first.first_published_at.month
        months = []

        # loop over years and months
        for y in range(year, fyear-1, -1):
            start, end = 12, 0
            if y == year:
                start = month
            if y == fyear:
                end = fmonth-1

            for m in range(start, end, -1):
                # months.append((y, m, [m]))
                months.append((y, m, calendar.month_name[m], BlogPage.objects.filter(
                    first_published_at__month=m, first_published_at__year=y).count(), BlogPage.objects.filter(first_published_at__year=y).count()))
        return months
        # return "OK Archives"

    def likePost(self):
        print("Likes=", self.blPoLikeCount)
        self.blPoLikeCount = self.blPoLikeCount+int(1)
        self.save()  # note: you may need to ensure you call save to update data

    def dislikePost(self):
        self.blPoLikeCount = self.blPoLikeCount - int(1)
        self.save()  # note: you may need to ensure you call save to update data

    def bookmarkPost(self, request):
        if UserFavBlog.objects.filter(blog=self, posted_by=request.user, isFav=True).exists():
            pass
        else:
            UserFavBlog(blog=self, posted_by=request.user, isFav=True).save()

    def serve(self, request):
        if request.method == 'POST':
            if request.POST.get('like'):
                self.likePost()
            if request.POST.get('dislike'):
                self.dislikePost()
            if request.POST.get('bookmark'):
                self.bookmarkPost(request)
        return super(BlogPage, self).serve(request)

    class Meta:
        verbose_name = "Blog Page"
        # sort=['first_published_at']


class BlogReLinks(Orderable, TagColor):
    blog = ParentalKey(BlogPage, related_name="blog_reLinks")
    reLink = models.ForeignKey(
        BlogPage, on_delete=models.SET_NULL, related_name='+', blank=True, null=True
    )
    panels = [
        FieldPanel('blog'),
        FieldPanel('reLink'),
        FieldRowPanel([
            NativeColorPanel('bgColor'),
            NativeColorPanel('fgColor'),

        ]),
    ]
    # def __str__(self):
    #     return str(self.reLink)

    # class Meta:
    # verbose_name_plural = "Sub Category"
    # verbose_name = "Sub Category"
    # ordering = ['-id']


@register_snippet
class BlogComments(Orderable):
    blog = ParentalKey(BlogPage, related_name="blog_comments")
    comment = RichTextField(blank=True, null=True)
    like_count = models.PositiveIntegerField(blank=True, null=True)
    comIPaddr = models.CharField(max_length=200, blank=True, null=True)
    isApproved = models.BooleanField(blank=True, null=True, default=False)
    createdAt = models.DateTimeField(auto_now_add=True)
    publishedAt = models.DateTimeField(auto_now=True)
    posted_by = models.ForeignKey(
        User, on_delete=models.SET_NULL, null=True, blank=True)
    parent = models.ForeignKey(
        'self', on_delete=models.SET_NULL, null=True, blank=True, related_name="comment_replie")

    def __str__(self):
        return f'{self.comment}'

    panels = [
        FieldPanel('parent'),
        FieldPanel('comment'),
        MultiFieldPanel([
            FieldPanel('like_count'),
            FieldPanel('comIPaddr'),
            FieldPanel('isApproved'),

        ]),
        FieldPanel('posted_by'),
    ]

    class Meta:
        verbose_name_plural = "Blog Comment"
        ordering = ["-createdAt"]


@register_snippet
class UserFavBlog(models.Model):
    blog = models.ForeignKey(BlogPage, on_delete=models.SET_NULL, null=True)
    isFav = models.BooleanField(blank=True, null=True, default=False)
    user = models.ForeignKey(
        User, on_delete=models.SET_NULL, null=True, blank=True, related_name="fav_blogs")
    createdOn = models.DateTimeField(auto_now_add=True)

    def __str__(self):
        return f'{self.blog}'

    class Meta:
        verbose_name_plural = "User Fav Blog"
        ordering = ['-createdOn']


# Article End

# News
class NewsPage(MetadataPageMixin,Page, HitCountMixin):
    blPoTitleTemplate = models.CharField(max_length=200, blank=True, null=True, choices=[(
        'Single Row', 'Single Row'), ('Multi Row', 'Multi Row')], verbose_name="Title Template", default="Multi Row")
    customTitle=RichTextField(blank=True, null=True,verbose_name="Custom Title")
    
    titleImg = models.ForeignKey(
        'wagtailimages.Image', on_delete=models.PROTECT, related_name='+', blank=True, null=True, verbose_name="Title Image"
    )
    shortDesc = RichTextField(blank=True, null=True,
                              verbose_name="Short Description")

    blCategory = StreamField([
        ('category', ModelChooserBlock(Category)),
    ], use_json_field=True, blank=True, null=True)

    blbrands = StreamField([
        ('brand', ModelChooserBlock(Brand)),
    ], use_json_field=True, blank=True, null=True)

    product = models.ForeignKey(
        Product, on_delete=models.SET_NULL, null=True, blank=True)

    tags = ClusterTaggableManager(through=BlogPageTag, blank=True)
    products = StreamField([
        ('blog', PageChooserBlock('Blog.BlogPage')),
        ('product', ModelChooserBlock(Product)),
    ], use_json_field=True, blank=True, null=True)

    body = StreamField([
        ('carousel', CarouselBlock()),
        ('section', SectionBlock()),
        ('row', RowBlock()),

    ], use_json_field=True, blank=True, null=True)
    blPoBanner = StreamField([
        ('banner', CarouselBlock()),
    ], use_json_field=True, blank=True, null=True, max_num=1)

    blogStory = StreamField([
        ('web_story', WebStoryBlock()),
    ], use_json_field=True, blank=True, null=True, max_num=1)

    blogSocial = StreamField([
        ('social_links', BlogSocialBlock()),
    ], use_json_field=True, blank=True, null=True)
    blogGallery = StreamField([
        ('gallery', BlogGalleryBlock()),
    ], use_json_field=True, blank=True, null=True)
    blPoReadingTime = models.PositiveIntegerField(
        blank=True, null=True, default=2)
    showFooter = models.BooleanField(
        blank=True, null=True, default=False, verbose_name="Show Footer")
    blPoCommentLock = models.BooleanField(blank=True, null=True, default=False)
    blPoCommentLockMsg = models.CharField(
        max_length=200, blank=True, null=True)
    blPoRelatedTemplate = models.CharField(max_length=200, blank=True, null=True, choices=[(
        'Links', 'Links'), ('Img One Column', 'Img One Column'), ('Img Two Column', 'Img Two Column')], default="Img Two Column")
    content_panels = Page.content_panels + [
        FieldPanel('customTitle'),
        FieldPanel('blCategory', heading="Category"),
        FieldPanel('blbrands', heading="Brands"),
        FieldPanel('products'),
        MultiFieldPanel([
            FieldRowPanel([
                FieldPanel('blPoTitleTemplate'),
                # FieldPanel('blProduct'),
                # FieldPanel('product'),
                FieldPanel('titleImg'),
            ])
        ]),
        # FieldPanel('catVersion'),
        FieldPanel('shortDesc'),
        FieldPanel('tags'),
        FieldPanel('body'),
    ]
    promote_panels = Page.promote_panels + [
        FieldPanel('blPoCommentLock', heading="Lock Comment"),
        FieldPanel('blPoCommentLockMsg', heading="Message on Lock Comment"),
        FieldPanel('blPoReadingTime', heading="Reading Time"),
        FieldPanel('showFooter'),
    ]
    banner_panels = [
        FieldPanel('blPoBanner', heading="Banner")
    ]
    story_panels = [
        FieldPanel('blogStory', heading="Web Story")
    ]
    social_panels = [
        FieldPanel('blogSocial', heading="Social")
    ]
    gallery_panels = [
        FieldPanel('blogGallery', heading="Gallery")
    ]
    comments_panels = [
        InlinePanel('news_comments', label="Comments")
    ]
    re_panels = [
        FieldPanel('blPoRelatedTemplate', heading="Template"),

        InlinePanel('news_reLinks', label="Related Link")
    ]
    edit_handler = TabbedInterface([
        ObjectList(content_panels, heading='Content'),
        ObjectList(promote_panels, heading='Promote'),
        ObjectList(banner_panels, heading='Banner'),
        ObjectList(story_panels, heading='Story'),
        ObjectList(social_panels, heading='Social'),
        ObjectList(gallery_panels, heading='Gallery'),
        ObjectList(re_panels, heading='Related'),
        ObjectList(comments_panels, heading='Comments'),
        # ObjectList(Page.settings_panels, heading='Settings'), # The default settings are now displayed in the sidebar but need to be in the `TabbedInterface`.
    ])
    search_fields = Page.search_fields + [  # Inherit search_fields from Page
        index.SearchField('title'),
        # index.SearchField('category'),
        index.SearchField('tags'),
        index.SearchField('body'),
        index.FilterField('title'),
        # index.FilterField('category'),
        index.FilterField('tags'),
        index.FilterField('body'),
    ]

    def __str__(self):
        return str(self.title)

    def get_cat(self):
        print("Category=", self.blCategory)
        # self.blPoLikeCount = self.blPoLikeCount+int(1)
        # self.save()  # note: you may need to ensure you call save to update data

    def archives(self):
        """Make a list of months to show archive links."""
        if not BlogPage.objects.count():
            return []

        # set up vars
        year, month = time.localtime()[:2]
        first = BlogPage.objects.order_by("first_published_at")[0]
        fyear = first.first_published_at.year
        fmonth = first.first_published_at.month
        months = []

        # loop over years and months
        for y in range(year, fyear-1, -1):
            start, end = 12, 0
            if y == year:
                start = month
            if y == fyear:
                end = fmonth-1

            for m in range(start, end, -1):
                # months.append((y, m, [m]))
                months.append((y, m, calendar.month_name[m], BlogPage.objects.filter(
                    first_published_at__month=m, first_published_at__year=y).count(), BlogPage.objects.filter(first_published_at__year=y).count()))
        return months
        # return "OK Archives"

    def likePost(self):
        print("Likes=", self.blPoLikeCount)
        self.blPoLikeCount = self.blPoLikeCount+int(1)
        self.save()  # note: you may need to ensure you call save to update data

    def dislikePost(self):
        self.blPoLikeCount = self.blPoLikeCount - int(1)
        self.save()  # note: you may need to ensure you call save to update data

    def bookmarkPost(self, request):
        if UserFavBlog.objects.filter(blog=self, posted_by=request.user, isFav=True).exists():
            pass
        else:
            UserFavBlog(blog=self, posted_by=request.user, isFav=True).save()

    def serve(self, request):
        if request.method == 'POST':
            if request.POST.get('like'):
                self.likePost()
            if request.POST.get('dislike'):
                self.dislikePost()
            if request.POST.get('bookmark'):
                self.bookmarkPost(request)
        return super(BlogPage, self).serve(request)

    class Meta:
        verbose_name = "Blog Page"
        # sort=['first_published_at']


class NewsReLinks(Orderable, TagColor):
    news = ParentalKey(NewsPage, related_name="news_reLinks")
    reLink = models.ForeignKey(
        NewsPage, on_delete=models.SET_NULL, related_name='+', blank=True, null=True
    )
    panels = [
        FieldPanel('news'),
        FieldPanel('reLink'),
        FieldRowPanel([
            NativeColorPanel('bgColor'),
            NativeColorPanel('fgColor'),

        ]),
    ]


class NewsComments(Orderable):
    news = ParentalKey(NewsPage, related_name="news_comments")
    comment = RichTextField(blank=True, null=True)
    like_count = models.PositiveIntegerField(blank=True, null=True)
    comIPaddr = models.CharField(max_length=200, blank=True, null=True)
    isApproved = models.BooleanField(blank=True, null=True, default=False)
    createdAt = models.DateTimeField(auto_now_add=True)
    publishedAt = models.DateTimeField(auto_now=True)
    posted_by = models.ForeignKey(
        User, on_delete=models.SET_NULL, null=True, blank=True)
    parent = models.ForeignKey(
        'self', on_delete=models.SET_NULL, null=True, blank=True, related_name="comment_replie")

    def __str__(self):
        return f'{self.comment}'

    panels = [
        FieldPanel('parent'),
        FieldPanel('comment'),
        MultiFieldPanel([
            FieldPanel('like_count'),
            FieldPanel('comIPaddr'),
            FieldPanel('isApproved'),

        ]),
        FieldPanel('posted_by'),
    ]

    class Meta:
        verbose_name_plural = "News Comment"
        ordering = ["-createdAt"]


class UserFavNews(models.Model):
    news = models.ForeignKey(NewsPage, on_delete=models.SET_NULL, null=True)
    isFav = models.BooleanField(blank=True, null=True, default=False)
    user = models.ForeignKey(
        User, on_delete=models.SET_NULL, null=True, blank=True, related_name="fav_news")
    createdOn = models.DateTimeField(auto_now_add=True)

    def __str__(self):
        return f'{self.blog}'

    class Meta:
        verbose_name_plural = "User Fav News"
        ordering = ['-createdOn']


# News End

# Tutorial
class TutorialPage(MetadataPageMixin,Page, HitCountMixin):
    blPoTitleTemplate = models.CharField(max_length=200, blank=True, null=True, choices=[(
        'Single Row', 'Single Row'), ('Multi Row', 'Multi Row')], verbose_name="Title Template", default="Multi Row")
    customTitle=RichTextField(blank=True, null=True,verbose_name="Custom Title")
    
    titleImg = models.ForeignKey(
        'wagtailimages.Image', on_delete=models.PROTECT, related_name='+', blank=True, null=True, verbose_name="Title Image"
    )
    shortDesc = RichTextField(blank=True, null=True,
                              verbose_name="Short Description")

    blCategory = StreamField([
        ('category', ModelChooserBlock(Category)),
    ], use_json_field=True, blank=True, null=True)

    blbrands = StreamField([
        ('brand', ModelChooserBlock(Brand)),
    ], use_json_field=True, blank=True, null=True)

    # blSubCategory = StreamField([
    #     ('category', ModelChooserBlock(CategoryPage)),
    # ], use_json_field=True, blank=True, null=True)

    product = models.ForeignKey(
        Product, on_delete=models.SET_NULL, null=True, blank=True)

    # tags = ClusterTaggableManager(through=BlogPageTag, blank=True)
    products = StreamField([
        ('blog', PageChooserBlock('Blog.BlogPage')),
        ('product', ModelChooserBlock(Product)),
    ], use_json_field=True, blank=True, null=True)

    body = StreamField([
        ('carousel', CarouselBlock()),
        ('section', SectionBlock()),
        ('row', RowBlock()),

    ], use_json_field=True, blank=True, null=True)
    blPoBanner = StreamField([
        ('banner', CarouselBlock()),
    ], use_json_field=True, blank=True, null=True, max_num=1)

    blogStory = StreamField([
        ('web_story', WebStoryBlock()),
    ], use_json_field=True, blank=True, null=True, max_num=1)

    blogSocial = StreamField([
        ('social_links', BlogSocialBlock()),
    ], use_json_field=True, blank=True, null=True)
    blogGallery = StreamField([
        ('gallery', BlogGalleryBlock()),
    ], use_json_field=True, blank=True, null=True)

    blPoReadingTime = models.PositiveIntegerField(
        blank=True, null=True, default=2)
    showFooter = models.BooleanField(
        blank=True, null=True, default=False, verbose_name="Show Footer")
    blPoCommentLock = models.BooleanField(blank=True, null=True, default=False)
    blPoCommentLockMsg = models.CharField(
        max_length=200, blank=True, null=True)
    blPoRelatedTemplate = models.CharField(max_length=200, blank=True, null=True, choices=[(
        'Links', 'Links'), ('Img One Column', 'Img One Column'), ('Img Two Column', 'Img Two Column')], default="Img Two Column")
    content_panels = Page.content_panels + [
        FieldPanel('customTitle'),
        FieldPanel('blCategory', heading="Category"),
        FieldPanel('blbrands', heading="Brands"),
        FieldPanel('products'),
        MultiFieldPanel([
            FieldRowPanel([
                FieldPanel('blPoTitleTemplate'),
                # FieldPanel('blProduct'),
                # FieldPanel('product'),
                FieldPanel('titleImg'),
            ])
        ]),
        # FieldPanel('catVersion'),
        FieldPanel('shortDesc'),
        FieldPanel('body'),
    ]
    promote_panels = Page.promote_panels + [
        FieldPanel('blPoCommentLock', heading="Lock Comment"),
        FieldPanel('blPoCommentLockMsg', heading="Message on Lock Comment"),
        FieldPanel('blPoReadingTime', heading="Reading Time"),
        FieldPanel('showFooter'),
    ]
    banner_panels = [
        FieldPanel('blPoBanner', heading="Banner")
    ]
    story_panels = [
        FieldPanel('blogStory', heading="Web Story")
    ]
    social_panels = [
        FieldPanel('blogSocial', heading="Social")
    ]
    gallery_panels = [
        FieldPanel('blogGallery', heading="Gallery")
    ]
    comments_panels = [
        InlinePanel('tut_comments', label="Comments")
    ]
    re_panels = [
        FieldPanel('blPoRelatedTemplate', heading="Template"),

        InlinePanel('tut_reLinks', label="Related Link")
    ]
    edit_handler = TabbedInterface([
        ObjectList(content_panels, heading='Content'),
        ObjectList(promote_panels, heading='Promote'),
        ObjectList(banner_panels, heading='Banner'),
        ObjectList(story_panels, heading='Story'),
        ObjectList(social_panels, heading='Social'),
        ObjectList(gallery_panels, heading='Gallery'),
        ObjectList(re_panels, heading='Related'),
        ObjectList(comments_panels, heading='Comments'),
        # ObjectList(Page.settings_panels, heading='Settings'), # The default settings are now displayed in the sidebar but need to be in the `TabbedInterface`.
    ])
    search_fields = Page.search_fields + [  # Inherit search_fields from Page
        index.SearchField('title'),
        index.SearchField('body'),
        index.FilterField('title'),
        index.FilterField('body'),
    ]

    def __str__(self):
        return str(self.title)

    def get_cat(self):
        print("Category=", self.blCategory)
        # self.blPoLikeCount = self.blPoLikeCount+int(1)
        # self.save()  # note: you may need to ensure you call save to update data

    def archives(self):
        """Make a list of months to show archive links."""
        if not BlogPage.objects.count():
            return []

        # set up vars
        year, month = time.localtime()[:2]
        first = BlogPage.objects.order_by("first_published_at")[0]
        fyear = first.first_published_at.year
        fmonth = first.first_published_at.month
        months = []

        # loop over years and months
        for y in range(year, fyear-1, -1):
            start, end = 12, 0
            if y == year:
                start = month
            if y == fyear:
                end = fmonth-1

            for m in range(start, end, -1):
                # months.append((y, m, [m]))
                months.append((y, m, calendar.month_name[m], BlogPage.objects.filter(
                    first_published_at__month=m, first_published_at__year=y).count(), BlogPage.objects.filter(first_published_at__year=y).count()))
        return months
        # return "OK Archives"

    def likePost(self):
        print("Likes=", self.blPoLikeCount)
        self.blPoLikeCount = self.blPoLikeCount+int(1)
        self.save()  # note: you may need to ensure you call save to update data

    def dislikePost(self):
        self.blPoLikeCount = self.blPoLikeCount - int(1)
        self.save()  # note: you may need to ensure you call save to update data

    def bookmarkPost(self, request):
        if UserFavBlog.objects.filter(blog=self, posted_by=request.user, isFav=True).exists():
            pass
        else:
            UserFavBlog(blog=self, posted_by=request.user, isFav=True).save()

    # def serve(self, request):
    #     if request.method == 'POST':
    #         if request.POST.get('like'):
    #             self.likePost()
    #         if request.POST.get('dislike'):
    #             self.dislikePost()
    #         if request.POST.get('bookmark'):
    #             self.bookmarkPost(request)
    #     return super(BlogPage, self).serve(request)

    class Meta:
        verbose_name = "Tutorial Page"
        # sort=['first_published_at']


class TutReLinks(Orderable, TagColor):
    tut = ParentalKey(TutorialPage, related_name="tut_reLinks")
    reLink = models.ForeignKey(
        TutorialPage, on_delete=models.SET_NULL, related_name='+', blank=True, null=True
    )
    panels = [
        FieldPanel('tut'),
        FieldPanel('reLink'),
        FieldRowPanel([
            NativeColorPanel('bgColor'),
            NativeColorPanel('fgColor'),

        ]),
    ]


class TutComments(Orderable):
    blog = ParentalKey(TutorialPage, related_name="tut_comments")
    comment = RichTextField(blank=True, null=True)
    like_count = models.PositiveIntegerField(blank=True, null=True)
    comIPaddr = models.CharField(max_length=200, blank=True, null=True)
    isApproved = models.BooleanField(blank=True, null=True, default=False)
    createdAt = models.DateTimeField(auto_now_add=True)
    publishedAt = models.DateTimeField(auto_now=True)
    posted_by = models.ForeignKey(
        User, on_delete=models.SET_NULL, null=True, blank=True)
    parent = models.ForeignKey(
        'self', on_delete=models.SET_NULL, null=True, blank=True, related_name="comment_replie")

    def __str__(self):
        return f'{self.comment}'

    panels = [
        FieldPanel('parent'),
        FieldPanel('comment'),
        MultiFieldPanel([
            FieldPanel('like_count'),
            FieldPanel('comIPaddr'),
            FieldPanel('isApproved'),

        ]),
        FieldPanel('posted_by'),
    ]

    class Meta:
        verbose_name_plural = "Tutorial Comment"
        ordering = ["-createdAt"]


class UserFavTut(models.Model):
    blog = models.ForeignKey(BlogPage, on_delete=models.SET_NULL, null=True)
    isFav = models.BooleanField(blank=True, null=True, default=False)
    user = models.ForeignKey(
        User, on_delete=models.SET_NULL, null=True, blank=True, related_name="fav_tut")
    createdOn = models.DateTimeField(auto_now_add=True)

    def __str__(self):
        return f'{self.blog}'

    class Meta:
        verbose_name_plural = "User Fav Tutorial"
        ordering = ['-createdOn']

# Tutorial End
